@@ -464,11 +464,13 @@ typedef enum
 	BUILTIN_FSHR,
 	BUILTIN_GATHER,
 	BUILTIN_GET_ROUNDING_MODE,
+	BUILTIN_INT_TO_MASK,
 	BUILTIN_LOG,
 	BUILTIN_LOG10,
 	BUILTIN_LOG2,
 	BUILTIN_MATRIX_MUL,
 	BUILTIN_MATRIX_TRANSPOSE,
+	BUILTIN_MASK_TO_INT,
 	BUILTIN_MASKED_LOAD,
 	BUILTIN_MASKED_STORE,
 	BUILTIN_MAX,
@@ -686,6 +688,8 @@ typedef enum
 	DECL_FNTYPE,
 	DECL_FUNC,
 	DECL_GROUP,
+	DECL_GENERIC,
+	DECL_GENERIC_INSTANCE,
 	DECL_IMPORT,
 	DECL_LABEL,
 	DECL_MACRO,
@@ -1752,6 +1756,13 @@ typedef enum
 	PARAM_RW_EXPAND_ELEMENTS,
 } ParamRewrite;
 
+typedef enum
+{
+	CT_EVAL_TYPE,
+	CT_EVAL_IDENTIFIER,
+	CT_EVAL_IMPLICIT_IDENTIFIER
+} CtEvalKind;
+
 // -- Arch helper macros
 #define ARCH_UNSUPPORTED ARCH_TYPE_AARCH64_32: case ARCH_TYPE_BPFEL: case ARCH_TYPE_BPFEB: case ARCH_TYPE_SPARCEL: \
  case ARCH_TYPE_LE64: case ARCH_TYPE_AMDIL: case ARCH_TYPE_AMDIL64: case ARCH_TYPE_HSAIL: case ARCH_TYPE_HSAIL64:  \
@@ -1774,7 +1785,7 @@ typedef enum
 	case DECL_ALIAS: case DECL_CT_ASSERT: case DECL_CT_EXEC: case DECL_FAULT: \
 	case DECL_CT_ECHO: case DECL_CT_INCLUDE: case DECL_GROUP: \
 	case DECL_BODYPARAM: case DECL_VAR: case DECL_ENUM_CONSTANT: \
-	case DECL_POISONED: case DECL_ALIAS_PATH
+	case DECL_POISONED: case DECL_ALIAS_PATH: case DECL_GENERIC: case DECL_GENERIC_INSTANCE
 
 // -- Expr helper macros
 #define NON_RUNTIME_EXPR EXPR_POISONED: \
