@@ -349,6 +349,10 @@ EXIT:;
 		}
 		return add_error_token(lexer, "An identifier may not consist of only '_' characters.");
 	}
+	if (len > MAX_IDENTIFIER_LENGTH)
+	{
+		return add_error_token(lexer, "An identifier cannot be longer than %d characters, but this one was %u characters long.", MAX_IDENTIFIER_LENGTH, len);
+	}
 	const char* interned_string = symtab_add(lexer->lexing_start, len, hash, &type);
 	switch (type)
 	{
@@ -858,9 +862,8 @@ static int append_esc_string_token(char *restrict dest, const char *restrict src
 			if (h < 0) return -1;
 			int l = char_hex_to_nibble(src[2]);
 			if (l < 0) return -1;
-			unicode_char = ((unsigned) h << 4U) + (unsigned)l;
-			scanned = 3;
-			break;
+			dest[(*pos)++] = (char)(((unsigned) h << 4U) + (unsigned)l);
+			return 3;
 		}
 		case 'u':
 		{
