@@ -184,6 +184,7 @@ static void linker_setup_windows(const char ***args_ref, Linker linker_type, con
 	{
 		if (crt_linking == WIN_CRT_STATIC)
 		{
+			// This path is now unreachable due to the check in compiler.c
 			add_concat_file_arg(compiler_path, "c3c_rt/clang_rt.asan-x86_64.lib");
 		}
 		else
@@ -295,9 +296,9 @@ static void linker_setup_macos(const char ***args_ref, Linker linker_type)
 }
 
 
-static const char *find_freebsd_crt(void)
+static const char *find_bsd_crt(void)
 {
-	if (file_exists("/usr/lib/crt1.o"))
+	if (file_exists("/usr/lib/crt1.o") || file_exists("/usr/lib/crt0.o"))
 	{
 		return "/usr/lib/";
 	}
@@ -402,12 +403,10 @@ static const char *find_linux_crt_begin(void)
 
 static const char *find_linux_ld(void)
 {
-	INFO_LOG("Environment Type ID: %d", compiler.platform.environment_type);
-	switch (compiler.platform.environment_type)
+	if (compiler.platform.environment_type == ENV_TYPE_ANDROID) return "--dynamic-linker=/system/ld-android.so";
+	switch (compiler.build.linuxpaths.libc)
 	{
-		case ENV_TYPE_MUSL:
-		case ENV_TYPE_MUSLEABI:
-		case ENV_TYPE_MUSLEABIHF:
+		case LINUX_LIBC_MUSL:
 			switch (compiler.platform.arch)
 			{
 				case ARCH_TYPE_ARM: return "--dynamic-linker=/lib/ld-musl-arm.so.1";
@@ -428,10 +427,9 @@ static const char *find_linux_ld(void)
 			}
 			UNREACHABLE;
 			break;
-		case ENV_TYPE_ANDROID:
-			return "--dynamic-linker=/system/ld-android.so";
-		default:
-			switch (compiler.platform.arch) {
+		case LINUX_LIBC_GNU:
+			switch (compiler.platform.arch)
+			{
 				case ARCH_TYPE_ARM: return "--dynamic-linker=/lib/ld-linux.so.3";
 				case ARCH_TYPE_AARCH64: return "--dynamic-linker=/lib/ld-linux-aarch64.so.1";
 				case ARCH_TYPE_MIPS: return "--dynamic-linker=/lib/ld-linux-mipsn8.so.1";
@@ -443,11 +441,12 @@ static const char *find_linux_ld(void)
 				case ARCH_TYPE_SPARCV9: return "--dynamic-linker=/lib/ld-linux.so.2";
 				case ARCH_TYPE_X86: return "--dynamic-linker=/lib64/ld-linux.so.2";
 				case ARCH_TYPE_X86_64: return "--dynamic-linker=/lib64/ld-linux-x86-64.so.2";
-				default: return "--dynamic-linkrt=/lib/ld-linux-unknown.so.2"; // another placeholder until we have all of them
+				default: return "--dynamic-linker=/lib/ld-linux-unknown.so.2"; // another placeholder until we have all of them
 			}
+			FALLTHROUGH;
+		default:
 			UNREACHABLE;
 	}
-	UNREACHABLE;
 }
 
 static void linker_setup_linux(const char ***args_ref, Linker linker_type, bool is_dylib)
@@ -585,58 +584,78 @@ static void linker_setup_android(const char ***args_ref, Linker linker_type, boo
 	add_plain_arg("-lc");
 }
 
-static void linker_setup_freebsd(const char ***args_ref, Linker linker_type, bool is_dylib)
+static void linker_setup_bsd(const char ***args_ref, Linker linker_type, bool is_dylib)
 {
 	if (linker_type == LINKER_CC)
 	{
 		if (compiler.linking.link_math) linking_add_link(&compiler.linking, "m");
 		linking_add_link(&compiler.linking, "pthread");
-		linking_add_link(&compiler.linking, "execinfo"); // for backtrace
-		if (compiler.build.debug_info == DEBUG_INFO_FULL)
-		{
-			add_plain_arg("-rdynamic");
-		}
+		linking_add_link(&compiler.linking, "execinfo");
+		if (compiler.build.debug_info == DEBUG_INFO_FULL) add_plain_arg("-rdynamic");
 		return;
 	}
 	if (is_no_pie(compiler.platform.reloc_model)) add_plain_arg("-no-pie");
-	if (is_pie(compiler.platform.reloc_model)) add_plain_arg("-pie");
 	add_plain_arg("--eh-frame-hdr");
-
 	if (!link_libc()) return;
-
-	const char *crt_dir = find_freebsd_crt();
-	if (!crt_dir)
+	const char *crt_dir = find_bsd_crt();
+	if (!crt_dir) error_exit("Failed to find the C runtime at link time.");
+	if (strip_unused() && compiler.build.type == TARGET_TYPE_EXECUTABLE) add_plain_arg("--gc-sections");
+	bool is_openbsd = compiler.platform.os == OS_TYPE_OPENBSD;
+	bool is_netbsd = compiler.platform.os == OS_TYPE_NETBSD;
+	bool is_pie_pic_mode = is_pie_pic(compiler.platform.reloc_model);
+	if (is_openbsd)
 	{
-		error_exit("Failed to find the C runtime at link time.");
-	}
-	if (strip_unused() && compiler.build.type == TARGET_TYPE_EXECUTABLE)
-	{
-		add_plain_arg("--gc-sections");
-	}
-	if (is_pie_pic(compiler.platform.reloc_model))
-	{
-		add_plain_arg("-pie");
-		add_concat_file_arg(crt_dir, "crti.o");
-		if (!is_dylib) add_concat_file_arg(crt_dir, "Scrt1.o");
-		add_concat_file_arg(crt_dir, "crtbeginS.o");
-		add_concat_file_arg(crt_dir, "crtendS.o");
-	}
-	else
-	{
-		add_concat_file_arg(crt_dir, "crti.o");
-		if (!is_dylib) add_concat_file_arg(crt_dir, "crt1.o");
+		if (!is_dylib) add_concat_file_arg(crt_dir, "crt0.o");
 		add_concat_file_arg(crt_dir, "crtbegin.o");
 		add_concat_file_arg(crt_dir, "crtend.o");
 	}
-	add_concat_file_arg(crt_dir, "crtn.o");
+	else
+	{
+		if (!is_openbsd) add_concat_file_arg(crt_dir, "crti.o");
+		if (is_dylib || is_pie_pic_mode)
+		{
+			if (!is_dylib && is_pie(compiler.platform.reloc_model)) add_plain_arg("-pie");
+			if (!is_dylib) add_concat_file_arg(crt_dir, is_netbsd ? "crt0.o" : "Scrt1.o");
+			add_concat_file_arg(crt_dir, "crtbeginS.o");
+			add_concat_file_arg(crt_dir, "crtendS.o");
+		}
+		else
+		{
+			if (!is_dylib) add_concat_file_arg(crt_dir, is_netbsd ? "crt0.o" : "crt1.o");
+			add_concat_file_arg(crt_dir, "crtbegin.o");
+			add_concat_file_arg(crt_dir, "crtend.o");
+		}
+		if (!is_openbsd) add_concat_file_arg(crt_dir, "crtn.o");
+	}
 	add_concat_quote_arg("-L", crt_dir);
-	add_plain_arg("--dynamic-linker=/libexec/ld-elf.so.1");
-	linking_add_link(&compiler.linking, "c");
-	if (compiler.linking.link_math) linking_add_link(&compiler.linking, "m");
-	linking_add_link(&compiler.linking, "gcc");
-	linking_add_link(&compiler.linking, "gcc_s");
-
 	add_plain_arg("-L/usr/lib/");
+	switch (compiler.platform.os)
+	{
+		case OS_TYPE_NETBSD:
+			add_plain_arg("--dynamic-linker=/usr/libexec/ld.elf_so");
+			if (is_dylib)
+			{
+				add_plain_arg("--no-rosegment");
+				add_plain_arg("-znorelro");
+			}
+			break;
+		case OS_TYPE_OPENBSD:
+			add_plain_arg("--dynamic-linker=/usr/libexec/ld.so");
+			break;
+		default:
+			add_plain_arg("--dynamic-linker=/libexec/ld-elf.so.1");
+	}
+	linking_add_link(&compiler.linking, "c");
+	if (is_openbsd)
+	{
+		linking_add_link(&compiler.linking, "execinfo");
+	}
+	else
+	{
+		linking_add_link(&compiler.linking, "gcc");
+		linking_add_link(&compiler.linking, "gcc_s");
+	}
+	if (compiler.linking.link_math) linking_add_link(&compiler.linking, "m");
 	add_plain_arg("-m");
 	add_plain_arg(ld_target(compiler.platform.arch));
 }
@@ -707,6 +726,7 @@ static bool linker_setup(const char ***args_ref, const char **files_to_link, uns
 			if (is_dylib)
 			{
 				add_plain_arg("/DLL");
+				add_concat_quote_arg("/IMPLIB:", static_lib_name());
 			}
 			else
 			{
@@ -737,7 +757,7 @@ static bool linker_setup(const char ***args_ref, const char **files_to_link, uns
 		case OS_TYPE_FREEBSD:
 		case OS_TYPE_OPENBSD:
 		case OS_TYPE_NETBSD:
-			linker_setup_freebsd(args_ref, linker_type, is_dylib);
+			linker_setup_bsd(args_ref, linker_type, is_dylib);
 			break;
 		case OS_TYPE_LINUX:
 			linker_setup_linux(args_ref, linker_type, is_dylib);
@@ -923,8 +943,8 @@ static bool link_exe(const char *output_file, const char **files_to_link, unsign
 			UNREACHABLE
 	}
 #else 
-    success = false;
-    error = "linking (.exe) is not implemented for C3C compiled without LLVM";
+	success = false;
+	error = "linking (.exe) is not implemented for C3C compiled without LLVM";
 #endif 
 	if (!success)
 	{
@@ -1154,9 +1174,9 @@ bool dynamic_lib_linker(const char *output_file, const char **files, unsigned fi
 		return true;
 	}
 	bool success;
-    const char *error = NULL;
+	const char *error = NULL;
 #if LLVM_AVAILABLE
-    int count = assemble_link_arguments(args, vec_size(args));
+	int count = assemble_link_arguments(args, vec_size(args));
 	switch (compiler.platform.object_format)
 	{
 		case OBJ_FORMAT_COFF:
@@ -1175,8 +1195,8 @@ bool dynamic_lib_linker(const char *output_file, const char **files, unsigned fi
 			UNREACHABLE
 	}
 #else 
-    success = false;
-    error = "linking not implemented for c3c compiled without llvm";
+	success = false;
+	error = "linking not implemented for c3c compiled without llvm";
 #endif 
 	if (!success)
 	{
@@ -1210,7 +1230,7 @@ bool static_lib_linker(const char *output_file, const char **files, unsigned fil
 	}
 	return llvm_ar(output_file, files, file_count, format);
 #else 
-    return false;
+	return false;
 #endif 
 }
 
