@@ -171,7 +171,18 @@ INLINE void llvm_emit_unaligned_store(GenContext *c, BEValue *result_value, Expr
 	llvm_emit_expr(c, &value, expr->call_expr.arguments[0]);
 	llvm_value_rvalue(c, &value);
 	llvm_emit_expr(c, result_value, expr->call_expr.arguments[1]);
-	llvm_store_to_ptr_aligned(c, value.value, result_value, expr->call_expr.arguments[2]->const_expr.ixx.i.low);
+	LLVMValueRef store = llvm_store_to_ptr_aligned(c, value.value, result_value, expr->call_expr.arguments[2]->const_expr.ixx.i.low);
+	if (store && expr->call_expr.arguments[3]->const_expr.b)
+	{
+		if (LLVMIsAMemCpyInst(store))
+		{
+			LLVMSetOperand(store, 3, LLVMConstAllOnes(c->bool_type));
+		}
+		else
+		{
+			LLVMSetVolatile(store, true);
+		}
+	}
 	c->emitting_load_store_check = emit_check;
 }
 
@@ -253,9 +264,11 @@ INLINE void llvm_emit_unaligned_load(GenContext *c, BEValue *result_value, Expr
 	bool emit_check = c->emitting_load_store_check;
 	c->emitting_load_store_check = true;
 	llvm_emit_expr(c, result_value, expr->call_expr.arguments[0]);
+	bool is_volatile = expr->call_expr.arguments[2]->const_expr.b;
 	llvm_value_deref(c, result_value);
 	result_value->alignment = expr->call_expr.arguments[1]->const_expr.ixx.i.low;
 	llvm_value_rvalue(c, result_value);
+	if (is_volatile && result_value->value) LLVMSetVolatile(result_value->value, is_volatile);
 	c->emitting_load_store_check = emit_check;
 }
 
@@ -595,6 +608,35 @@ static void llvm_emit_gather(GenContext *c, BEValue *be_value, Expr *expr)
 	llvm_value_set(be_value, result, expr->type);
 }
 
+static void llvm_emit_mask_to_int(GenContext *c, BEValue *be_value, Expr *expr)
+{
+	Expr **args = expr->call_expr.arguments;
+	ASSERT(vec_size(args) == 1);
+	LLVMValueRef val = llvm_emit_expr_to_rvalue(c, args[0]);
+	LLVMTypeRef mask_type = LLVMTypeOf(val);
+	unsigned bits = LLVMGetVectorSize(mask_type);
+	val = LLVMBuildBitCast(c->builder, val, LLVMIntTypeInContext(c->context, bits), "");
+	unsigned target_bits = next_highest_power_of_2(bits);
+	if (target_bits < 8) target_bits = 8;
+	if (target_bits < bits) val = LLVMBuildZExt(c->builder, val, llvm_get_type(c, expr->type), "");
+	llvm_value_set(be_value, val, expr->type);
+}
+
+static void llvm_emit_int_to_mask(GenContext *c, BEValue *be_value, Expr *expr)
+{
+	Expr **args = expr->call_expr.arguments;
+	ASSERT(vec_size(args) == 2);
+	LLVMValueRef val = llvm_emit_expr_to_rvalue(c, args[0]);
+	unsigned bits = (unsigned)args[1]->const_expr.ixx.i.low;
+	unsigned int_len = type_bit_size(args[0]->type);
+	if (int_len > bits)
+	{
+		val = LLVMBuildTrunc(c->builder, val, LLVMIntTypeInContext(c->context, bits), "");
+	}
+	val = LLVMBuildBitCast(c->builder, val, LLVMVectorType(c->bool_type, bits), "");
+	llvm_value_set(be_value, val, expr->type);
+}
+
 static void llvm_emit_masked_store(GenContext *c, BEValue *be_value, Expr *expr)
 {
 	Expr **args = expr->call_expr.arguments;
@@ -665,7 +707,7 @@ static void llvm_emit_overflow_builtin(GenContext *c, BEValue *be_value, Expr *e
 	// Note that we can make additional improvements here!
 	llvm_value_set_address(c, &ref, ref.value, ref.type->pointer, type_abi_alignment(ref.type->pointer));
 	LLVMTypeRef call_type[1] = { LLVMTypeOf(arg_slots[0]) };
-	unsigned intrinsic = type_is_signed(type_lowering(args[0]->type)) ? intrinsic_signed : intrinsic_unsigned;
+	unsigned intrinsic = type_is_signed_any(type_lowering(args[0]->type)) ? intrinsic_signed : intrinsic_unsigned;
 	LLVMValueRef result = llvm_emit_call_intrinsic(c, intrinsic, call_type, 1, arg_slots, 2);
 	LLVMValueRef failed = llvm_emit_extract_value(c, result, 1);
 	LLVMValueRef value = llvm_emit_extract_value(c, result, 0);
@@ -697,7 +739,7 @@ static void llvm_emit_wrap_builtin(GenContext *c, BEValue *result_value, Expr *e
 			res = LLVMBuildMul(c->builder, arg_slots[0], arg_slots[1], "emul");
 			break;
 		case BUILTIN_EXACT_DIV:
-			if (type_is_signed(base_type))
+			if (type_is_signed_any(base_type))
 			{
 				res = LLVMBuildSDiv(c->builder, arg_slots[0], arg_slots[1], "esdiv");
 			}
@@ -707,7 +749,7 @@ static void llvm_emit_wrap_builtin(GenContext *c, BEValue *result_value, Expr *e
 			}
 			break;
 		case BUILTIN_EXACT_MOD:
-			if (type_is_signed(base_type))
+			if (type_is_signed_any(base_type))
 			{
 				res = LLVMBuildSRem(c->builder, arg_slots[0], arg_slots[1], "eumod");
 			}
@@ -915,6 +957,12 @@ void llvm_emit_builtin_call(GenContext *c, BEValue *result_value, Expr *expr)
 		case BUILTIN_SCATTER:
 			llvm_emit_scatter(c, result_value, expr);
 			return;
+		case BUILTIN_MASK_TO_INT:
+			llvm_emit_mask_to_int(c, result_value, expr);
+			return;
+		case BUILTIN_INT_TO_MASK:
+			llvm_emit_int_to_mask(c, result_value, expr);
+			return;
 		case BUILTIN_MASKED_STORE:
 			llvm_emit_masked_store(c, result_value, expr);
 			return;
