@@ -716,6 +716,22 @@ static Expr *parse_unary_expr(ParseContext *c, Expr *left, SourceSpan lhs_start
 	return unary;
 }
 
+static Expr *parse_raise_expr_suffix(ParseContext *c, Expr *left_side, SourceSpan lhs_start)
+{
+	ASSERT(expr_ok(left_side));
+	if (left_side->expr_kind == EXPR_TYPEINFO)
+	{
+		PRINT_ERROR_AT(left_side, "Expected a fault before '~'.");
+		return poisoned_expr;
+	}
+	advance_and_verify(c, TOKEN_BIT_NOT);
+	Expr *expr = expr_new(EXPR_OPTIONAL, lhs_start);
+	expr->inner_expr = left_side;
+	RANGE_EXTEND_PREV(expr);
+	return expr;
+}
+
+
 /**
  * post_unary_expr ::= <expr> unary_op
  */
@@ -770,6 +786,7 @@ static Expr *parse_ternary_expr(ParseContext *c, Expr *left_side, SourceSpan lhs
 		expr->expr_kind = EXPR_OPTIONAL;
 		expr->inner_expr = left_side;
 		RANGE_EXTEND_PREV(expr);
+		SEMA_DEPRECATED(expr, "Using '?' to create an optional is deprecated, use '~' instead.");
 		return expr;
 	}
 
@@ -868,6 +885,7 @@ static Expr *parse_initializer_list(ParseContext *c, Expr *left, SourceSpan lhs_
 			if (i == 0 && expr->expr_kind == EXPR_SPLAT)
 			{
 				splat = expr;
+				continue;
 			}
 			if (expr->expr_kind == EXPR_DESIGNATOR)
 			{
@@ -876,12 +894,10 @@ static Expr *parse_initializer_list(ParseContext *c, Expr *left, SourceSpan lhs_
 					designated = expr->designator_expr.path[0]->kind == DESIGNATOR_FIELD ? 1 : 2;
 					goto ERROR;
 				}
-
 				designated = expr->designator_expr.path[0]->kind == DESIGNATOR_FIELD ? 1 : 2;
 				continue;
 			}
 			if (designated > 0) goto ERROR;
-			if (designated == -1 && splat) continue;
 			designated = 0;
 			continue;
 ERROR:;
@@ -1176,7 +1192,7 @@ static Expr *parse_ct_defined(ParseContext *c, Expr *left, SourceSpan lhs_start
 		vec_add(defined->expression_list, expr);
 		if (!try_consume(c, TOKEN_COMMA))
 		{
-			CONSUME_OR_RET(TOKEN_RPAREN, false);
+			CONSUME_OR_RET(TOKEN_RPAREN, poisoned_expr);
 			break;
 		}
 	}
@@ -1634,7 +1650,7 @@ EXIT:
 	Type *type_base = NULL;
 	if (type_bits)
 	{
-		if (type_bits < 0 || !is_power_of_two((uint64_t)type_bits) || type_bits > 128)
+		if (type_bits < 8 || !is_power_of_two((uint64_t)type_bits) || type_bits > 128)
 		{
 			PRINT_ERROR_AT(expr_int, "Integer type suffix should be i8, i16, i32, i64 or i128.");
 			return poisoned_expr;
@@ -2196,7 +2212,7 @@ ParseRule rules[TOKEN_EOF + 1] = {
 		[TOKEN_DOT] = { NULL, parse_access_expr, PREC_CALL },
 		[TOKEN_BANG] = { parse_unary_expr, parse_rethrow_expr, PREC_CALL },
 		[TOKEN_BYTES] = { parse_bytes_expr, NULL, PREC_NONE },
-		[TOKEN_BIT_NOT] = { parse_unary_expr, NULL, PREC_UNARY },
+		[TOKEN_BIT_NOT] = { parse_unary_expr, parse_raise_expr_suffix, PREC_CALL },
 		[TOKEN_BIT_XOR] = { NULL, parse_binary, PREC_BIT },
 		[TOKEN_BIT_OR] = { NULL, parse_binary, PREC_BIT },
 		[TOKEN_AMP] = { parse_unary_expr, parse_binary, PREC_BIT },
