@@ -72,13 +72,14 @@ static inline void parse_translation_unit(ParseContext *c)
 		if (!decl) continue;
 		if (decl_ok(decl))
 		{
+			Decl *gs = decl->is_template ? declptr(decl->generic_id) : NULL;
 			if (decl->is_cond)
 			{
-				add_decl_to_list(&c->unit->global_cond_decls, decl);
+				add_decl_to_list(gs ? &gs->generic_decl.conditional_decls : &c->unit->global_cond_decls, decl);
 			}
 			else
 			{
-				add_decl_to_list(&c->unit->global_decls, decl);
+				add_decl_to_list(gs ? &gs->generic_decl.decls : &c->unit->global_decls, decl);
 			}
 		}
 		else
