@@ -47,6 +47,7 @@ Ast *parse_short_body(ParseContext *c, TypeInfoId return_type, bool is_regular_f
 bool parse_attribute(ParseContext *c, Attr **attribute_ref, bool expect_eos);
 
 bool parse_attributes(ParseContext *c, Attr ***attributes_ref, Visibility *visibility_ref, bool *builtin_ref, bool *cond_ref);
+Decl *parse_generic_decl(ParseContext *c);
 
 bool parse_switch_body(ParseContext *c, Ast ***cases, TokenType case_type, TokenType default_type);
 Expr *parse_ct_expression_list(ParseContext *c, bool allow_decl);
@@ -61,6 +62,7 @@ bool parse_parameters(ParseContext *c, Decl ***params_ref,
 
 bool parse_arg_list(ParseContext *c, Expr ***result, TokenType param_end, bool vasplat);
 Expr *parse_type_compound_literal_expr_after_type(ParseContext *c, TypeInfo *type_info);
+void parse_attach_generics(ParseContext *c, Decl *generic_decl);
 
 INLINE void add_decl_to_list(Decl ***list, Decl *decl)
 {
