@@ -37,7 +37,7 @@ static bool sema_expr_analyse_syscall(SemaContext *context, Expr *expr);
 static inline bool sema_expr_analyse_swizzle(SemaContext *context, Expr *expr, bool swizzle_two);
 static inline int builtin_expected_args(BuiltinFunction func);
 static inline bool is_valid_atomicity(SemaContext *context, Expr *expr);
-static bool sema_check_alignment_expression(SemaContext *context, Expr *align);
+static bool sema_check_alignment_expression(SemaContext *context, Expr *align, bool may_be_zero);
 
 static bool sema_expr_is_valid_mask_for_value(SemaContext *context, Expr *expr, Expr *value)
 {
@@ -78,15 +78,16 @@ static bool sema_check_builtin_args_const(SemaContext *context, Expr **args, siz
 	return true;
 }
 
-static bool sema_check_alignment_expression(SemaContext *context, Expr *align)
+static bool sema_check_alignment_expression(SemaContext *context, Expr *align, bool may_be_zero)
 {
 	if (!sema_analyse_expr_rhs(context, type_usz, align, false, NULL, false)) return false;
 	if (!expr_is_const_int(align)
 	    || !int_fits(align->const_expr.ixx, TYPE_U64)
 	    || (!is_power_of_two(align->const_expr.ixx.i.low) && align->const_expr.ixx.i.low))
 	{
-		RETURN_SEMA_ERROR(align, "Expected a constant power-of-two alignment or zero.");
+		RETURN_SEMA_ERROR(align, may_be_zero ? "Expected a constant power-of-two alignment or zero." : "Expected a constant power-of-two alignment.");
 	}
+	if (!may_be_zero && align->const_expr.ixx.i.low == 0) RETURN_SEMA_ERROR(align, "Alignment must not be zero.");
 	return true;
 }
 
@@ -258,7 +259,7 @@ static bool sema_expr_analyse_compare_exchange(SemaContext *context, Expr *expr)
 		RETURN_SEMA_ERROR(args[6], "Failure ordering may not be RELEASE / ACQUIRE_RELEASE.");
 	}
 	Expr *align = args[7];
-	if (!sema_check_alignment_expression(context, align)) return false;
+	if (!sema_check_alignment_expression(context, align, true)) return false;
 	expr->type = type_add_optional(args[1]->type, optional);
 	return true;
 }
@@ -395,7 +396,7 @@ bool sema_expr_analyse_str_find(SemaContext *context, Expr *expr)
 bool sema_expr_analyse_str_conv(SemaContext *context, Expr *expr, BuiltinFunction func)
 {
 	Expr *inner = expr->call_expr.arguments[0];
-	if (!sema_analyse_expr_rvalue(context, inner)) return true;
+	if (!sema_analyse_expr_rvalue(context, inner)) return false;
 	if (!expr_is_const_string(inner))
 	{
 		RETURN_SEMA_ERROR(inner, "You need a compile time constant string to take convert.");
@@ -550,7 +551,7 @@ bool sema_expr_analyse_str_wide(SemaContext *context, Expr *expr, BuiltinFunctio
 		zero_terminate = zero_term->const_expr.b;
 	}
 	if (!sema_analyse_expr_rvalue(context, inner)) return false;
-	if (!sema_cast_const(inner) && !expr_is_const_string(inner))
+	if (!sema_cast_const(inner) || !expr_is_const_string(inner))
 	{
 		RETURN_SEMA_ERROR(inner, "You need a compile time constant string to convert to a wide string.");
 	}
@@ -640,6 +641,14 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 		RETURN_SEMA_ERROR(args[expected_args], "Too many arguments.");
 	}
 
+	for (unsigned i = 0; i < arg_count; i++)
+	{
+		if (args[i]->expr_kind == EXPR_NAMED_ARGUMENT)
+		{
+			RETURN_SEMA_ERROR(args[i], "Named arguments are not allowed in builtin calls.");
+		}
+	}
+
 	switch (func)
 	{
 		case BUILTIN_SPRINTF:
@@ -835,6 +844,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_VEC, BA_INTEGER, BA_INTEGER}, 3)) return false;
 			if (!sema_check_builtin_args_const(context, &args[1], 2)) return false;
 			ArraySize vec_len = type_flatten(args[0]->type)->array.len;
+			if (!cast_implicit(context, args[2], args[1]->type, false)) return false;
 			Int sum = int_mul(args[1]->const_expr.ixx, args[2]->const_expr.ixx);
 			if (!int_icomp(sum, vec_len, BINARYOP_EQ))
 			{
@@ -1040,7 +1050,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 								  type_get_vector(pointer_type->pointer, flat_pointer_vec->type_kind, len)),
 						  type_quoted_error_string(args[2]->type));
 			}
-			if (!sema_check_alignment_expression(context, args[3])) return false;
+			if (!sema_check_alignment_expression(context, args[3], true)) return false;
 			if (!sema_expr_is_valid_mask_for_value(context, args[1], args[2])) return false;
 			rtype = type_get_vector(pointer_type->pointer, flat_pointer_vec->type_kind, len);
 			break;
@@ -1063,11 +1073,49 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 								  type_get_vector_from_vector(pointer_type->pointer, flat_pointer_vec)),
 						  type_quoted_error_string(args[2]->type));
 			}
-			if (!sema_check_alignment_expression(context, args[3])) return false;
+			if (!sema_check_alignment_expression(context, args[3], true)) return false;
 			if (!sema_expr_is_valid_mask_for_value(context, args[2], args[1])) return false;
 			rtype = type_void;
 			break;
 		}
+		case BUILTIN_INT_TO_MASK:
+		{
+			ASSERT(arg_count == 2);
+			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_INTEGER, BA_INTEGER }, 2)) return false;
+			Expr *len = args[1];
+			if (!sema_cast_const(len) || !expr_is_const_int(len))
+			{
+				RETURN_SEMA_ERROR(len, "Expected constant integer for the vector length.");
+			}
+			Type *type = type_flatten(args[0]->type);
+			int size = (int)type_bit_size(type);
+			if (int_icomp(len->const_expr.ixx, size, BINARYOP_GT))
+			{
+				RETURN_SEMA_ERROR(args[1], "The vector length (%s) cannot be greater than the bit width of the integer (%d).",
+					int_to_str(len->const_expr.ixx, 10, false), size);
+			}
+			int bits = (int)len->const_expr.ixx.i.low;
+			if (!bits)
+			{
+				RETURN_SEMA_ERROR(args[1], "The vector length cannot be zero");
+			}
+			rtype = type_get_vector(type_bool, TYPE_VECTOR, bits);
+			break;
+		}
+		case BUILTIN_MASK_TO_INT:
+		{
+			ASSERT(arg_count == 1);
+			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_BOOLVEC }, 1)) return false;
+			Type *vec = type_flatten(args[0]->type);
+			ArraySize len = vec->array.len;
+			if (len > 128)
+			{
+				RETURN_SEMA_ERROR(args[0], "Masks must be 128 or fewer bits to convert them to an integer.");
+			}
+			if (len < 8) len = 8;
+			rtype = type_int_unsigned_by_bitsize(next_highest_power_of_2(len));
+			break;
+		}
 		case BUILTIN_MASKED_LOAD:
 		{
 			ASSERT(arg_count == 4);
@@ -1076,9 +1124,9 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!type_is_pointer(pointer_type)) RETURN_SEMA_ERROR(args[0], "Expected a direct pointer.");
 			if (pointer_type->pointer->canonical != args[2]->type->canonical)
 			{
-				RETURN_SEMA_ERROR(args[2], "Expected the value to be of type '%s'.", type_quoted_error_string(pointer_type->pointer));
+				RETURN_SEMA_ERROR(args[2], "Expected the value to be of type %s.", type_quoted_error_string(pointer_type->pointer));
 			}
-			if (!sema_check_alignment_expression(context, args[3])) return false;
+			if (!sema_check_alignment_expression(context, args[3], true)) return false;
 			if (!sema_expr_is_valid_mask_for_value(context, args[1], args[2])) return false;
 			rtype = pointer_type->pointer;
 			break;
@@ -1093,7 +1141,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			{
 				RETURN_SEMA_ERROR(args[2], "Expected the value to be of type %s.", type_quoted_error_string(pointer_type->pointer));
 			}
-			if (!sema_check_alignment_expression(context, args[3])) return false;
+			if (!sema_check_alignment_expression(context, args[3], true)) return false;
 			if (!sema_expr_is_valid_mask_for_value(context, args[2], args[1])) return false;
 			rtype = type_void;
 			break;
@@ -1166,21 +1214,23 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 		}
 		case BUILTIN_UNALIGNED_LOAD:
 		{
-			ASSERT(arg_count == 2);
-			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_POINTER, BA_INTEGER}, 2)) return false;
+			ASSERT(arg_count == 3);
+			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_POINTER, BA_INTEGER, BA_BOOL}, 3)) return false;
 			Type *original = type_flatten(args[0]->type);
 			if (original == type_voidptr) RETURN_SEMA_ERROR(args[0], "Expected a typed pointer.");
-			if (!sema_check_alignment_expression(context, args[1])) return false;
+			if (!sema_check_alignment_expression(context, args[1], false)) return false;
+			if (!sema_cast_const(args[2])) RETURN_SEMA_ERROR(args[2], "'is_volatile' must be a compile time constant.");
 			rtype = original->pointer;
 			break;
 		}
 		case BUILTIN_UNALIGNED_STORE:
 		{
-			ASSERT(arg_count == 3);
+			ASSERT(arg_count == 4);
 			if (!sema_check_builtin_args(context, args, (BuiltinArg[]) {BA_POINTER}, 1)) return false;
-			if (!sema_check_builtin_args(context, &args[2], (BuiltinArg[]) {BA_INTEGER}, 1)) return false;
+			if (!sema_check_builtin_args(context, &args[2], (BuiltinArg[]) {BA_INTEGER, BA_BOOL}, 2)) return false;
 			Type *original = type_flatten(args[0]->type);
-			if (!sema_check_alignment_expression(context, args[2])) return false;
+			if (!sema_check_alignment_expression(context, args[2], false)) return false;
+			if (!sema_cast_const(args[3])) RETURN_SEMA_ERROR(args[3], "'is_volatile' must be a compile time constant.");
 			if (original != type_voidptr)
 			{
 				if (!cast_implicit(context, args[1], original->pointer, false)) return false;
@@ -1233,7 +1283,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!sema_cast_const(args[3])) RETURN_SEMA_ERROR(args[3], "Ordering must be a compile time constant.");
 			if (!is_valid_atomicity(context, args[3])) return false;
 			if (args[3]->const_expr.ixx.i.low == ATOMIC_UNORDERED) RETURN_SEMA_ERROR(args[3], "'unordered' is not valid ordering.");
-			if (!sema_check_alignment_expression(context, args[4])) return false;
+			if (!sema_check_alignment_expression(context, args[4], true)) return false;
 			rtype = args[1]->type;
 			break;
 		}
@@ -1253,7 +1303,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!sema_cast_const(args[3])) RETURN_SEMA_ERROR(args[3], "Ordering must be a compile time constant.");
 			if (!is_valid_atomicity(context, args[3])) return false;
 			if (args[3]->const_expr.ixx.i.low == ATOMIC_UNORDERED) RETURN_SEMA_ERROR(args[3], "'unordered' is not valid ordering.");
-			if (!sema_check_alignment_expression(context, args[4])) return false;
+			if (!sema_check_alignment_expression(context, args[4], true)) return false;
 			rtype = args[1]->type;
 			break;
 		}
@@ -1272,7 +1322,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!sema_cast_const(args[3])) RETURN_SEMA_ERROR(args[3], "Ordering must be a compile time constant.");
 			if (!is_valid_atomicity(context, args[3])) return false;
 			if (args[3]->const_expr.ixx.i.low == ATOMIC_UNORDERED) RETURN_SEMA_ERROR(args[3], "'unordered' is not valid ordering.");
-			if (!sema_check_alignment_expression(context, args[4])) return false;
+			if (!sema_check_alignment_expression(context, args[4], true)) return false;
 			rtype = args[1]->type;
 			break;
 		}
@@ -1292,7 +1342,7 @@ bool sema_expr_analyse_builtin_call(SemaContext *context, Expr *expr)
 			if (!sema_cast_const(args[3])) RETURN_SEMA_ERROR(args[3], "Ordering must be a compile time constant.");
 			if (!is_valid_atomicity(context, args[3])) return false;
 			if (args[3]->const_expr.ixx.i.low == ATOMIC_UNORDERED) RETURN_SEMA_ERROR(args[3], "'unordered' is not valid ordering.");
-			if (!sema_check_alignment_expression(context, args[4])) return false;
+			if (!sema_check_alignment_expression(context, args[4], true)) return false;
 			rtype = args[1]->type;
 			break;
 		}
@@ -1376,6 +1426,7 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_LOG:
 		case BUILTIN_LRINT:
 		case BUILTIN_LROUND:
+		case BUILTIN_MASK_TO_INT:
 		case BUILTIN_NEARBYINT:
 		case BUILTIN_POPCOUNT:
 		case BUILTIN_REDUCE_ADD:
@@ -1394,15 +1445,16 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_SIN:
 		case BUILTIN_SQRT:
 		case BUILTIN_STR_HASH:
-		case BUILTIN_STR_UPPER:
 		case BUILTIN_STR_LOWER:
-		case BUILTIN_STR_SNAKECASE:
 		case BUILTIN_STR_PASCALCASE:
+		case BUILTIN_STR_SNAKECASE:
+		case BUILTIN_STR_UPPER:
 		case BUILTIN_TRUNC:
 		case BUILTIN_VOLATILE_LOAD:
 		case BUILTIN_WASM_MEMORY_SIZE:
 			return 1;
-		case BUILTIN_STR_FIND:
+		case BUILTIN_VOLATILE_STORE:
+		case BUILTIN_ANY_MAKE:
 		case BUILTIN_COPYSIGN:
 		case BUILTIN_EXACT_ADD:
 		case BUILTIN_EXACT_DIV:
@@ -1410,6 +1462,7 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_EXACT_MUL:
 		case BUILTIN_EXACT_SUB:
 		case BUILTIN_EXPECT:
+		case BUILTIN_INT_TO_MASK:
 		case BUILTIN_MAX:
 		case BUILTIN_MIN:
 		case BUILTIN_POW:
@@ -1417,19 +1470,17 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_REDUCE_FADD:
 		case BUILTIN_REDUCE_FMUL:
 		case BUILTIN_SAT_ADD:
+		case BUILTIN_SAT_MUL:
 		case BUILTIN_SAT_SHL:
 		case BUILTIN_SAT_SUB:
-		case BUILTIN_SAT_MUL:
-		case BUILTIN_VOLATILE_STORE:
-		case BUILTIN_VECCOMPNE:
-		case BUILTIN_VECCOMPLT:
-		case BUILTIN_VECCOMPLE:
+		case BUILTIN_STR_FIND:
+		case BUILTIN_VECCOMPEQ:
 		case BUILTIN_VECCOMPGE:
 		case BUILTIN_VECCOMPGT:
-		case BUILTIN_VECCOMPEQ:
+		case BUILTIN_VECCOMPLE:
+		case BUILTIN_VECCOMPLT:
+		case BUILTIN_VECCOMPNE:
 		case BUILTIN_WASM_MEMORY_GROW:
-		case BUILTIN_ANY_MAKE:
-		case BUILTIN_UNALIGNED_LOAD:
 			return 2;
 		case BUILTIN_EXPECT_WITH_PROBABILITY:
 		case BUILTIN_FMA:
@@ -1441,7 +1492,7 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_OVERFLOW_SUB:
 		case BUILTIN_PREFETCH:
 		case BUILTIN_ATOMIC_LOAD:
-		case BUILTIN_UNALIGNED_STORE:
+		case BUILTIN_UNALIGNED_LOAD:
 		case BUILTIN_SELECT:
 		case BUILTIN_MATRIX_TRANSPOSE:
 			return 3;
@@ -1451,6 +1502,7 @@ static inline int builtin_expected_args(BuiltinFunction func)
 		case BUILTIN_GATHER:
 		case BUILTIN_SCATTER:
 		case BUILTIN_STR_REPLACE:
+		case BUILTIN_UNALIGNED_STORE:
 			return 4;
 		case BUILTIN_ATOMIC_FETCH_EXCHANGE:
 		case BUILTIN_ATOMIC_FETCH_ADD:
