@@ -145,9 +145,12 @@ static bool sema_concat_bytes_and_other(SemaContext *context, Expr *expr, Expr *
 {
 	ArraySize len = left->const_expr.bytes.len;
 	bool is_bytes = left->const_expr.const_kind == CONST_BYTES;
+	ASSERT(is_bytes || left->const_expr.const_kind == CONST_STRING);
 	Type *indexed = type_get_indexed_type(left->type);
 	const char *left_bytes = left->const_expr.bytes.ptr;
-	RETRY:;
+RETRY:;
+	(void)sema_cast_const(right);
+	ASSERT(expr_is_const(right));
 	switch (right->const_expr.const_kind)
 	{
 		case CONST_INTEGER:
@@ -187,6 +190,10 @@ static bool sema_concat_bytes_and_other(SemaContext *context, Expr *expr, Expr *
 		case CONST_SLICE:
 		case CONST_INITIALIZER:
 			if (!cast_implicit(context, right, type_get_inferred_array(indexed), false)) return false;
+			if (!sema_cast_const(right))
+			{
+				RETURN_SEMA_ERROR(right, "Could not concatenate with the right hand side.");
+			}
 			expr_contract_array(&right->const_expr, left->const_expr.const_kind);
 			goto RETRY;
 	}
@@ -425,9 +432,20 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 					UNREACHABLE
 				case CONST_INIT_ARRAY_FULL:
 				{
+					Expr *el;
 					FOREACH(ConstInitializer *, val, init->init_array_full)
 					{
-						vec_add(untyped_exprs, val->init_value);
+						switch (val->kind)
+						{
+							case CONST_INIT_VALUE:
+								vec_add(untyped_exprs, val->init_value);
+								break;
+							default:
+								el = expr_new_expr(EXPR_CONST, single_expr);
+								expr_rewrite_const_initializer(el, val->type, val);
+								vec_add(untyped_exprs, el);
+								break;
+						}
 					}
 					continue;
 				}
