@@ -348,6 +348,10 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 		// Generate a nice error message for zero.
 		RETURN_SEMA_ERROR(elements[0], "Too many elements in initializer, it must be empty.");
 	}
+	if (expected_members > 0 && count > 0 && count != expected_members)
+	{
+		RETURN_SEMA_ERROR(elements[0], "Too %s elements in initializer, expected %u.", count > expected_members ? "many" : "few", expected_members);
+	}
 
 	bool optional = false;
 	bool is_vector = type_flat_is_vector(assigned);
@@ -515,13 +519,11 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 		}
 	}
 	Type *original = flattened->canonical;
-	bool is_bitstruct = original->type_kind == TYPE_BITSTRUCT;
-	bool is_structlike = type_is_union_or_strukt(original) || is_bitstruct;
+	bool is_structlike = type_is_union_or_strukt(original) || original->type_kind == TYPE_BITSTRUCT;
 	ArrayIndex max_index = -1;
 	bool optional = false;
 	Type *inner_type = NULL;
-	bool is_inferred = type_is_inferred(flattened);
-	int bitmember_count_without_value = 0;
+	bool is_inferred = type_is_infer_type(flattened);
 	FOREACH(Expr *, expr, init_expressions)
 	{
 		Decl *member;
@@ -529,11 +531,10 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 		if (!result) return false;
 		bool is_bitmember = member && member->decl_kind == DECL_VAR && member->var.kind == VARDECL_BITMEMBER;
 		Expr *value = expr->designator_expr.value;
-		if (!value && is_bitmember && member->var.start_bit == member->var.end_bit && type_flatten(result) == type_bool) {
-			ASSERT(is_bitstruct);
+		if (!value && is_bitmember && member->var.start_bit == member->var.end_bit && type_flatten(result) == type_bool)
+		{
 			value = expr_new_const_bool(INVALID_SPAN, type_bool, true);
 			expr->designator_expr.value = value;
-			bitmember_count_without_value += 1;
 		}
 		if (!value) RETURN_SEMA_ERROR(expr, "This initializer needs a value.");
 		if (!sema_analyse_expr_rhs(context, result, value, true, no_match_ref, false)) return false;
@@ -548,24 +549,17 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 			inner_type = type_no_optional(value->type);
 		}
 	}
-	if (bitmember_count_without_value != 0 && bitmember_count_without_value != vec_size(init_expressions))
-	{
-		RETURN_SEMA_ERROR(initializer, "Mixing the omission of initializers is not permitted.");
-	}
 	Type *type;
 	if (!is_structlike && is_inferred)
 	{
-		if (type_is_infer_type(flattened))
-		{
-			type = type_from_inferred(flattened, inner_type, (ArraySize)(max_index + 1));
-		}
-		else
-		{
-			type = type_from_inferred(flattened, inner_type, flattened->array.len);
-		}
+		type = type_from_inferred(flattened, flattened->array.base, (ArraySize)(max_index + 1));
 	}
 	else
 	{
+		if (!is_structlike && type_is_inferred(flattened))
+		{
+			RETURN_SEMA_ERROR(initializer, "Inferring size when having non-top inferrence is not supported.");
+		}
 		type = assigned;
 	}
 	if (splat && type->canonical != splat->type->canonical)
@@ -877,6 +871,10 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 		{
 			if (is_zero_init)
 			{
+				if (type_len_is_inferred(flattened->array.base))
+				{
+					RETURN_SEMA_ERROR(expr, "Inferring the slice inner type from an empty initializer is not possible.");
+				}
 				expr_rewrite_const_empty_slice(expr, to);
 				return true;
 			}
@@ -951,7 +949,7 @@ void const_init_rewrite_to_value(ConstInitializer *const_init, Expr *value)
 		Decl *ident = decl_flatten(value->ident_expr);
 		ASSERT(ident->decl_kind == DECL_VAR);
 		ASSERT(ident->var.kind == VARDECL_CONST);
-		const_init_rewrite_to_value(const_init, expr_copy(ident->var.init_expr));
+		const_init_rewrite_to_value(const_init, copy_expr_single(ident->var.init_expr));
 		return;
 	}
 	const_init->init_value = value;
@@ -1262,6 +1260,7 @@ static inline void sema_update_const_initializer_with_designator(
 
 static Type *sema_expr_analyse_designator(SemaContext *context, Type *current, Expr *expr, ArrayIndex *max_index, Decl **member_ptr)
 {
+	ASSERT(expr->expr_kind == EXPR_DESIGNATOR);
 	DesignatorElement **path = expr->designator_expr.path;
 
 	// Walk down into this path
@@ -1377,10 +1376,10 @@ static Type *sema_find_type_of_element(SemaContext *context, Type *type, Designa
 				*did_report_error = true;
 				return NULL;
 			}
-			if (end_index > (ArrayIndex)len)
+			if (end_index >= (ArrayIndex)len)
 			{
 				*did_report_error = true;
-				SEMA_ERROR(element->index_expr, "The index may must be less than the array length (which was %llu).", (unsigned long long)len);
+				SEMA_ERROR(element->index_end_expr, "The index must be less than the array length (which was %llu).", (unsigned long long)len);
 				return NULL;
 			}
 			element->index_end = end_index;
