@@ -53,7 +53,7 @@ void sema_analyse_pass_module_hierarchy(Module *module)
 	// No match, so we create a synthetic module.
 	Path *path = path_create_from_string(slice.ptr, slice.len, module->name->span);
 	DEBUG_LOG("Creating parent module for %s: %s", module->name->module, path->module);
-	Module *parent_module = compiler_find_or_create_module(path, NULL);
+	Module *parent_module = compiler_find_or_create_module(path);
 	module->parent_module = parent_module;
 	vec_add(parent_module->sub_modules, module);
 	sema_analyze_stage(parent_module, ANALYSIS_MODULE_HIERARCHY);
@@ -101,7 +101,7 @@ void sema_analysis_pass_process_imports(Module *module)
 				if (unit->if_attr)
 				{
 					unit->error_import = import;
-					import_module = compiler_find_or_create_module(path, NULL);
+					import_module = compiler_find_or_create_module(path);
 					goto FOUND_MODULE;
 				}
 				PRINT_ERROR_AT(import, "No module named '%s' could be found, did you type the name right?", path->module);
@@ -134,7 +134,7 @@ NEXT:;
 				if (unit->if_attr)
 				{
 					unit->error_import = alias_module;
-					import_module = compiler_find_or_create_module(path, NULL);
+					import_module = compiler_find_or_create_module(path);
 					goto FOUND_ALIAS;
 				}
 				PRINT_ERROR_AT(path, "No module named '%s' could be found, did you type the name right?", path->module);
@@ -163,6 +163,49 @@ FOUND_ALIAS:
 	DEBUG_LOG("Pass finished processing %d import(s) with %d error(s).", total_import_count, compiler.context.errors_found);
 }
 
+static bool sema_check_if_implicit_generic(SemaContext *context, Decl *decl)
+{
+	if (!decl->func_decl.type_parent) return false;
+	TypeInfo *typedecl = type_infoptr(decl->func_decl.type_parent);
+	Decl *d = NULL;
+	if (typedecl->resolve_status == RESOLVE_DONE)
+	{
+		CanonicalType *type = typedecl->type->canonical;
+		if (!type_is_user_defined(type)) return false;
+		d = type->decl;
+	}
+	else if (typedecl->kind == TYPE_INFO_IDENTIFIER && typedecl->subtype == TYPE_COMPRESSED_NONE)
+	{
+		d = sema_resolve_maybe_parameterized_symbol(context, typedecl->unresolved.name, typedecl->unresolved.path, typedecl->span);
+	}
+	return d && d->is_template;
+}
+
+void unit_register_optional_global_decl(CompilationUnit *unit, Decl *decl)
+{
+	SemaContext context;
+	sema_context_init(&context, unit);
+	if (decl->is_templated) context.generic_instance = declptr(decl->instance_id);
+	if (!decl->is_templated && (decl->decl_kind == DECL_MACRO || decl->decl_kind == DECL_FUNC))
+	{
+		if (sema_check_if_implicit_generic(&context, decl))
+		{
+			unit_register_global_decl(unit, decl);
+			sema_context_destroy(&context);
+			return;
+		}
+	}
+	if (sema_decl_if_cond(&context, decl))
+	{
+		unit_register_global_decl(unit, decl);
+	}
+	else
+	{
+		decl->decl_kind = DECL_ERASED;
+	}
+	sema_context_destroy(&context);
+
+}
 INLINE void register_global_decls(CompilationUnit *unit, Decl **decls)
 {
 	FOREACH(Decl *, decl, decls)
@@ -443,14 +486,16 @@ void sema_analysis_pass_process_methods(Module *module, bool process_generic)
 				vec_add(unit->generic_methods_to_register, method);
 				continue;
 			}
-			sema_analyse_method_register(&context, method);
-			if (method->decl_kind == DECL_MACRO)
+			if (sema_analyse_method_register(&context, method))
 			{
-				vec_add(unit->macro_methods, method);
-			}
-			else
-			{
-				vec_add(unit->methods, method);
+				if (method->decl_kind == DECL_MACRO)
+				{
+					vec_add(unit->macro_methods, method);
+				}
+				else
+				{
+					vec_add(unit->methods, method);
+				}
 			}
 		}
 		sema_context_destroy(&context);
@@ -467,6 +512,7 @@ void sema_analysis_pass_process_methods(Module *module, bool process_generic)
 	DEBUG_LOG("Pass finished with %d error(s).", compiler.context.errors_found);
 }
 
+
 void sema_analysis_pass_register_conditional_units(Module *module)
 {
 	DEBUG_LOG("Pass: Register conditional units for %s", module->name->module);
@@ -497,6 +543,15 @@ void sema_analysis_pass_register_conditional_units(Module *module)
 		{
 			vec_resize(unit->global_decls, 0);
 			vec_resize(unit->global_cond_decls, 0);
+			FOREACH(Decl *, decl, module->generic_sections)
+			{
+				if (decl->unit == unit)
+				{
+					vec_resize(decl->generic_decl.conditional_decls, 0);
+					vec_resize(decl->generic_decl.decls, 0);
+				}
+			}
+			vec_resize(unit->ct_includes, 0);
 			continue;
 		}
 CHECK_LINK:
@@ -565,13 +620,7 @@ RETRY:;
 		Decl **decls = unit->global_cond_decls;
 		FOREACH(Decl *, decl, decls)
 		{
-			SemaContext context;
-			sema_context_init(&context, unit);
-			if (sema_decl_if_cond(&context, decl))
-			{
-				unit_register_global_decl(unit, decl);
-			}
-			sema_context_destroy(&context);
+			unit_register_optional_global_decl(unit, decl);
 		}
 		vec_resize(decls, 0);
 RETRY_INCLUDES:
@@ -630,7 +679,7 @@ void sema_analysis_pass_ct_echo(Module *module)
 	DEBUG_LOG("Pass finished with %d error(s).", compiler.context.errors_found);
 }
 
-static inline bool analyse_func_body(SemaContext *context, Decl *decl)
+bool analyse_func_body(SemaContext *context, Decl *decl)
 {
 	if (!decl->func_decl.body) return true;
 	if (decl->is_extern)
@@ -648,7 +697,7 @@ static inline bool analyse_func_body(SemaContext *context, Decl *decl)
 	return true;
 }
 
-INLINE void sema_analyse_inner_func_ptr(SemaContext *c, Decl *decl)
+void sema_analyse_inner_func_ptr(SemaContext *c, Decl *decl)
 {
 	Type *inner;
 	switch (decl->decl_kind)
@@ -776,7 +825,7 @@ static bool sema_check_interface(SemaContext *context, Decl *decl, TypeInfo *int
 	}
 	return true;
 }
-static inline bool sema_check_interfaces(SemaContext *context, Decl *decl)
+bool sema_check_interfaces(SemaContext *context, Decl *decl)
 {
 	Decl **store = sema_decl_stack_store();
 	sema_add_methods_to_decl_stack(context, decl);
