@@ -611,7 +611,7 @@ static void cpu_features_set_to_features(CpuFeatures enabled_features, CpuFeatur
 		scratch_buffer_append(list[i]);
 		scratch_buffer_append_char(',');
 	}
-	if (scratch_buffer.len > 0) scratch_buffer.len--;
+	if (scratch_buffer.len > 0) scratch_buffer_delete(1);
 	compiler.platform.features = scratch_buffer_copy();
 }
 
@@ -1197,6 +1197,7 @@ static char *arch_to_target_triple(ArchOsTarget target, LinuxLibc linux_libc)
 		case MACOS_AARCH64: return "aarch64-apple-macosx";
 		case ELF_AARCH64: return "aarch64-unknown-elf";
 		case WINDOWS_AARCH64: return "aarch64-pc-windows-msvc";
+		case NETBSD_AARCH64: return "aarch64-unknown-netbsd";
 		case LINUX_RISCV32: return linux_libc == LINUX_LIBC_MUSL ? "riscv32-unknown-linux-musl" : "riscv32-unknown-linux";
 		case ELF_RISCV32: return "riscv32-unknown-elf";
 		case LINUX_RISCV64: return linux_libc == LINUX_LIBC_MUSL ? "riscv64-unknown-linux-musl" : "riscv64-unknown-linux";
@@ -1682,9 +1683,7 @@ static AlignData os_target_alignment_of_int(OsType os, ArchType arch, uint32_t b
 		case ARCH_TYPE_PPC64:
 		case ARCH_TYPE_PPC:
 		case ARCH_TYPE_PPC64LE:
-		case ARCH_TYPE_WASM64:
 		case ARCH_TYPE_RISCV32:
-		case ARCH_TYPE_WASM32:
 		case ARCH_TYPE_XTENSA:
 			return (AlignData) { MIN(64u, bits), MIN(64u, bits) };
 		case ARCH_TYPE_X86_64:
@@ -1695,6 +1694,13 @@ static AlignData os_target_alignment_of_int(OsType os, ArchType arch, uint32_t b
 #endif
 		case ARCH_TYPE_RISCV64:
 			return (AlignData) { bits, bits };
+		case ARCH_TYPE_WASM64:
+		case ARCH_TYPE_WASM32:
+#if LLVM_AVAILABLE && LLVM_VERSION_MAJOR < 20
+			return (AlignData) { MIN(64u, bits), MIN(64u, bits) };
+#else
+			return (AlignData) { bits, bits };
+#endif
 		case ARCH_TYPE_AARCH64:
 		case ARCH_TYPE_AARCH64_BE:
 			if (bits < 32) return (AlignData){ bits, 32 };
