@@ -172,6 +172,7 @@ static inline StringSlice slice_from_string(const char *data);
 void slice_trim(StringSlice *slice);
 
 void scratch_buffer_clear(void);
+void scratch_buffer_delete(size_t len);
 void scratch_buffer_append(const char *string);
 void scratch_buffer_append_len(const char *string, size_t len);
 bool scratch_buffer_may_append(size_t len);
@@ -182,7 +183,6 @@ void scratch_buffer_append_remove_space(const char *start, int len);
 void scratch_buffer_append_signed_int(int64_t i);
 void scratch_buffer_append_double(double d);
 void scratch_buffer_append_shell_escaped(const char *string);
-void scratch_buffer_append_but_mangle_underscore_dot(const char *name);
 void scratch_buffer_append_cmd_argument(const char *string);
 UNUSED void scratch_buffer_append_unsigned_int(uint64_t i);
 void scratch_buffer_printf(const char *format, ...);
@@ -610,7 +610,7 @@ static char hex_conv[256] = {
 
 static inline int char_hex_to_nibble(char c)
 {
-	return hex_conv[(unsigned)c] - 1;
+	return hex_conv[(unsigned char)c] - 1;
 }
 
 static inline bool char_is_hex_or_(char c)
